package com.example.centrosalud;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

import android.annotation.SuppressLint;
import android.content.res.Resources.NotFoundException;
import android.os.Build;
import android.os.Bundle;
import android.support.v4.app.FragmentActivity;
import android.util.Log;
import android.util.Xml;
import android.view.View;
import android.view.ViewTreeObserver.OnGlobalLayoutListener;

import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.LatLngBounds;
import com.google.android.gms.maps.model.MarkerOptions;

public class MainActivity extends FragmentActivity {

	private static final String ns = null;
	private GoogleMap mMap;
	private ArrayList<MarkerOptions> centros;

	@Override
	protected void onCreate(Bundle savedInstanceState) {

		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		centros = new ArrayList<MarkerOptions>();
		readXML();
		setUpMapIfNeeded();
	}

	@Override
	protected void onResume() {
		super.onResume();
		setUpMapIfNeeded();
	}

	/**
	 * Sets up the map if it is possible to do so (i.e., the Google Play
	 * services APK is correctly installed) and the map has not already been
	 * instantiated.. This will ensure that we only ever call
	 * {@link #setUpMap()} once when {@link #mMap} is not null.
	 * <p>
	 * If it isn't installed {@link SupportMapFragment} (and
	 * {@link com.google.android.gms.maps.MapView MapView}) will show a prompt
	 * for the user to install/update the Google Play services APK on their
	 * device.
	 * <p>
	 * A user can return to this Activity after following the prompt and
	 * correctly installing/updating/enabling the Google Play services. Since
	 * the Activity may not have been completely destroyed during this process
	 * (it is likely that it would only be stopped or paused),
	 * {@link #onCreate(Bundle)} may not be called again so we should call this
	 * method in {@link #onResume()} to guarantee that it will be called.
	 */
	private void setUpMapIfNeeded() {
		// Do a null check to confirm that we have not already instantiated the
		// map.
		if (mMap == null) {
			// Try to obtain the map from the SupportMapFragment.
			mMap = ((SupportMapFragment) getSupportFragmentManager()
					.findFragmentById(R.id.map)).getMap();
			// Check if we were successful in obtaining the map.
			if (mMap != null) {
				setUpMap();
			} else
				Log.d("Error", "No se pudo obtener el mapa");
		}
	}

	/**
	 * This is where we can add markers or lines, add listeners or move the
	 * camera. In this case, we just add a marker near Africa.
	 * <p>
	 * This should only be called once and when we are sure that {@link #mMap}
	 * is not null.
	 */
	private void setUpMap() {

		for (int i = 0; i < centros.size(); i++) {
			mMap.addMarker(centros.get(i));
		}

		// Pan to see all markers in view.
		// Cannot zoom to bounds until the map has a size.
		final View mapView = getSupportFragmentManager().findFragmentById(
				R.id.map).getView();
		if (mapView.getViewTreeObserver().isAlive()) {
			mapView.getViewTreeObserver().addOnGlobalLayoutListener(
					new OnGlobalLayoutListener() {
						@SuppressLint("NewApi")
						// We check which build version we are using.
						@Override
						public void onGlobalLayout() {
							LatLngBounds.Builder bounds = new LatLngBounds.Builder();
							for (int i = 0; i < centros.size(); i++)
								bounds.include(centros.get(i).getPosition());
							if (Build.VERSION.SDK_INT < Build.VERSION_CODES.JELLY_BEAN) {
								mapView.getViewTreeObserver()
										.removeGlobalOnLayoutListener(this);
							} else {
								mapView.getViewTreeObserver()
										.removeOnGlobalLayoutListener(this);
							}
							mMap.moveCamera(CameraUpdateFactory
									.newLatLngBounds(bounds.build(), 70));
						}
					});
		}
	}

	private void readXML() {
		try {
			parse(getResources().openRawResource(R.raw.centros));
		} catch (NotFoundException e) {
			// TODO Auto-generated catch block
			//Log.e("Error", "NotFoundException");
		} catch (XmlPullParserException e) {
			// TODO Auto-generated catch block
			//Log.e("Error", "XMLPullParser");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			//Log.e("Error", "IOError");
		}
	}

	private void parse(InputStream in) throws XmlPullParserException,
			IOException {
		try {
			XmlPullParser parser = Xml.newPullParser();
			parser.setFeature(XmlPullParser.FEATURE_PROCESS_NAMESPACES, false);
			parser.setInput(in, "UTF-8");
			parser.nextTag();
			//Log.i("PullParsing nextTag", "" + parser.nextTag());
			readGpoints(parser);
		} finally {
			in.close();
		}
	}

	private void readGpoints(XmlPullParser parser)
			throws XmlPullParserException, IOException {

		parser.require(XmlPullParser.START_TAG, ns, "gpoints");
		while (parser.next() != XmlPullParser.END_DOCUMENT) {
			//Log.d("pasa", "bucle");
			if (parser.getEventType() != XmlPullParser.START_TAG) {
				//Log.d("pasa", "entra");
				continue;
			}
			String name = parser.getName();
			// Starts by looking for the entry tag
			if (name.equals("gpoint")) {
				readGpoint(parser);
			} else {
				skip(parser);
			}
			//Log.d("pasa", "pasa");
		}
		//Log.d("pasa", "Sale del while");
	}

	private void readGpoint(XmlPullParser parser)
			throws XmlPullParserException, IOException {
		String link = "";
		parser.require(XmlPullParser.START_TAG, ns, "gpoint");
		String tag = parser.getName();
		String nombre = parser.getAttributeValue(null, "nombre");
		String direccion = parser.getAttributeValue(null, "direccion");
		String lat = parser.getAttributeValue(null, "lat");
		String lng = parser.getAttributeValue(null, "lng");
		String icon = parser.getAttributeValue(null, "icon");
		//Log.d("readGpoint", nombre + " " + direccion + " " + lat + " " + lng);
		if (icon.equals("CS")) {
			centros.add(new MarkerOptions()
					.position(
							new LatLng(Double.parseDouble(lat), Double
									.parseDouble(lng))).title(nombre)
					.snippet(direccion)
					.icon(BitmapDescriptorFactory.fromResource(R.drawable.cs)));
		} else {
			if (icon.equals("HP")) {
				centros.add(new MarkerOptions()
						.position(
								new LatLng(Double.parseDouble(lat), Double
										.parseDouble(lng)))
						.title(nombre)
						.snippet(direccion)
						.icon(BitmapDescriptorFactory
								.fromResource(R.drawable.hospital)));
			}
		}
	}

	private void skip(XmlPullParser parser) throws XmlPullParserException,
			IOException {
		if (parser.getEventType() != XmlPullParser.START_TAG) {
			throw new IllegalStateException();
		}
		int depth = 1;
		while (depth != 0) {
			switch (parser.next()) {
			case XmlPullParser.END_TAG:
				depth--;
				break;
			case XmlPullParser.START_TAG:
				depth++;
				break;
			}
		}
	}

}
